module ComplexField
  module SessionIndexer
    def generate_solr_document
      super.tap do |solr_doc|
        index_session(solr_doc)
      end
    end

    def index_session(solr_doc)
      complex_sessions = object.complex_subjects.map{|subject| subject.complex_sessions}.flatten
      solr_doc[Solrizer.solr_name('complex_session', :displayable)] = complex_sessions.to_json

      complex_sessions.each do |c|
        # session_titles
        vals = c.session_title
        fld_name = Solrizer.solr_name('session', :stored_searchable)
        solr_doc[fld_name] = [] unless solr_doc.include?(fld_name)
        solr_doc[fld_name] << vals
        solr_doc[fld_name] = solr_doc[fld_name].flatten.uniq
        fld_name = Solrizer.solr_name('session', :facetable)
        solr_doc[fld_name] = [] unless solr_doc.include?(fld_name)
        solr_doc[fld_name] << vals
        solr_doc[fld_name] = solr_doc[fld_name].flatten.uniq
        
        # date_recorded as date
        unless c.session_date_recorded.blank?
          vals = c.session_date_recorded.strftime("%Y-%m-%d")
          fld_name = Solrizer.solr_name('date_recorded', :dateable)
          solr_doc[fld_name] = [] unless solr_doc.include?(fld_name)
          begin
            dates_utc = vals.tr('０-９ａ-ｚＡ-Ｚ', '0-9a-zA-Z')
            dates_utc = (dates_utc.length <= 4 ? DateTime.strptime(dates_utc, '%Y').utc.iso8601 : DateTime.parse(dates_utc).utc.iso8601) unless vals.blank?
          rescue ArgumentError
            dates_utc = vals.tr('０-９ａ-ｚＡ-Ｚ', '0-9a-zA-Z')
            dates_utc = DateTime.parse("#{dates_utc}-01").utc.iso8601 unless vals.blank?
          end
          # as date
          solr_doc[fld_name] << dates_utc unless dates_utc.blank?
          solr_doc[fld_name].flatten!
          # Add years
          year_fld = Solrizer.solr_name("year_recorded", :facetable)
          years = DateTime.parse(dates_utc).strftime("%Y") unless dates_utc.blank?
          solr_doc[year_fld] = [] unless solr_doc.include?(year_fld)
          solr_doc[year_fld] << years unless years.blank?
          solr_doc[year_fld].flatten!
          # date as displayable
          fld_name = Solrizer.solr_name("date_recorded", :displayable)
          solr_doc[fld_name] = [] unless solr_doc.include?(fld_name)
          solr_doc[fld_name] << vals
          solr_doc[fld_name].flatten!
        end
      end
    end

    def self.session_facet_fields
      # solr fields that will be treated as facets
      fields = []
      fields << Solrizer.solr_name('session', :facetable)
      fields << Solrizer.solr_name('year_recorded', :facetable)
      fields << Solrizer.solr_name('date_recorded', :dateable)
      fields
    end

    def self.session_search_fields
      # solr fields that will be used for a search
      fields = []
      fields << Solrizer.solr_name('session_title', :symbol)
      fields << Solrizer.solr_name('session', :stored_searchable)
      fields << Solrizer.solr_name('date_recorded', :dateable)
      fields
    end

    def self.session_show_fields
      # solr fields that will be used to display results on the record page
      fields = []
      fields << Solrizer.solr_name('complex_session', :displayable)
      fields
    end
  end
end
