# frozen_string_literal: true

module Bulkrax
  class RubCsvEntry < CsvEntry
    def build_metadata
      raise StandardError, 'Record not found' if record.nil?
      raise StandardError, "Missing required elements, missing element(s) are: #{importerexporter.parser.missing_elements(keys_without_numbers(record.keys)).join(', ')}" unless importerexporter.parser.required_elements?(keys_without_numbers(record.keys))

      self.parsed_metadata = {}
      add_identifier
      add_ingested_metadata
      # TODO(alishaevn): remove the collections stuff entirely and only reference collections via the new parents code
      add_collections
      add_visibility
      add_metadata_for_model
      add_rights_statement
      add_local
      sanitize_controlled_uri_values!

      self.parsed_metadata
    end

    def add_metadata_for_model
      if factory_class == Collection
        add_collection_type_gid
      elsif factory_class == FileSet
        validate_presence_of_filepath!
        validate_existence_of_file
        validate_presence_of_parent!
      else
        files = map_files
        record['s3_remote_files'] = files
        parsed_metadata['s3_remote_files'] = files
        #add_file unless importerexporter.metadata_only?
        add_admin_set_id
      end
    end

    def self.read_data(bucket_with_prefix)
      raise StandardError, 'CSV path empty' if bucket_with_prefix.blank?
      bucket_name, prefix = bucket_with_prefix.split('/')
      s3 = S3StorageService.new
      s3.init_client

      raise ::StandardError, "Bucket #{bucket_name} does not exist" unless s3.bucket_exists?(bucket_name)

      meta_object_key = prefix.present? ? File.join(prefix, "metadata.csv") : "metadata.csv"

      raise ::StandardError, "Metadata file #{meta_object_key} is missing" unless s3.object_exists?(bucket_name, meta_object_key)

      presigned_url = s3.presigned_url_for_download(bucket_name, meta_object_key)
      metadata = URI.open(presigned_url).read
      CSV.parse(metadata,
        headers: true,
        header_converters: ->(h) { h.to_sym },
        encoding: 'utf-8')
    end


    def map_files
      s3 = S3StorageService.new
      s3.init_client
      bucket_name, prefix = parser.import_file_path.split('/')
      key_prefix = prefix.present? ? File.join(prefix, record['dataset_path'], "data") : File.join(record['dataset_path'], "data")
      object_keys = []
      s3_objects, size = s3.list_all_objects(bucket_name, key_prefix)

      s3_objects.each do |obj|
        if obj[:key] != "metadata.csv" && obj[:size] != 0
          object_keys << File.join(bucket_name, obj[:key])
        end
      end

      object_keys
    end
  end
end
